// Copyright 2014 Rouslan Solomakhin
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

library passphrase_constructor;

import 'dart:math';

/// Constructs passphrases from diceware words. Sample usage:
/// ```
/// import 'dart:math';
/// import 'package:passphrase_constructor/passphrase_constructor.dart';
///
/// void main()
///   var pass = new PassphraseConstructor(
///       ['hello', 'world'], ['*&!', ')(%'], ['123', '098'], new Random());
///   var size = 15;
///   var useSpecials = true;
///   var useNumbers = false;
///   var useUpperCase = true;
///   var passphrase = pass.construct(
///       size, useSpecials, useNumbers, useUpperCase);
///   print passphrase;
/// }
/// ```
class PassphraseConstructor {
  List<String> words;
  List<String> specials;
  List<String> numbers;
  var rng = new Random();

  /// All calls to [construct] will use the provided arguments as source of the
  /// data.
  PassphraseConstructor(final List<String> words,
                        final List<String> specials,
                        final List<String> numbers,
                        final Random rng) {
    this.words = words;
    this.specials = specials;
    this.numbers = numbers;
    this.rng = rng;
  }

  /// Constructs a passphrase of at most [size] characters.
  List<String> construct(int size,
                         bool useSpecials,
                         bool useNumbers,
                         bool useUpperCase) {
    var result_string_length = 0;

    if (useSpecials && !specials.isEmpty) {
      result_string_length += specials.first.length;
    }

    if (useNumbers && !numbers.isEmpty) {
      result_string_length += numbers.first.length;
    }

    var result = [];
    bool used_all_words = false;
    for (int i = 0;
         i < words.length && result_string_length + words[i].length <= size;
         ++i) {
      result.add(words[i]);
      result_string_length += result.last.length;
      used_all_words = i == words.length - 1;
    }

    if (useUpperCase && !result.isEmpty) {
      var word = rng.nextInt(result.length);
      var letter = rng.nextInt(result[word].length);
      var value = result[word];
      result[word] = value.substring(0, letter) + value[letter].toUpperCase() +
          value.substring(letter + 1);
    }

    if (useSpecials && !specials.isEmpty) {
      result.insert(rng.nextInt(result.length + 1), specials.first);
    }

    for (int i = useSpecials && !specials.isEmpty ? 1 : 0;
         used_all_words && i < specials.length &&
             result_string_length + specials[i].length <= size;
         ++i) {
      result.insert(rng.nextInt(result.length + 1), specials[i]);
      result_string_length += specials[i].length;
    }

    if (useNumbers && !numbers.isEmpty) {
      result.insert(rng.nextInt(result.length + 1), numbers.first);
    }

    for (int i = useNumbers && !numbers.isEmpty ? 1 : 0;
         used_all_words && i < numbers.length &&
             result_string_length + numbers[i].length <= size;
         ++i) {
      result.insert(rng.nextInt(result.length + 1), numbers[i]);
      result_string_length += numbers[i].length;
    }

    return result;
  }
}